@@ -1,23 +1,7 @@
-import sys
 import os
-import glob
-import StringIO
 import unittest
-import new
+from support import simplejson, html5lib_test_files
 
-try:
-    import simplejson
-except:
-    import re
-    class simplejson:
-        def load(f):
-            true, false = True, False
-            input=re.sub(r'(".*?(?<!\\)")',r'u\1',f.read().decode('utf-8'))
-            return eval(input)
-        load = staticmethod(load)
-
-
-import html5lib
 from html5lib.tokenizer import HTMLTokenizer
 from html5lib import constants
 
@@ -42,19 +26,17 @@ class TokenizerTestParser(object):
         return self.outputTokens
 
     def processDoctype(self, token):
-        self.outputTokens.append([u"DOCTYPE", token["name"], token["data"]])
+        self.outputTokens.append([u"DOCTYPE", token["name"], token["publicId"], token["systemId"], token["correct"]])
 
     def processStartTag(self, token):
-        self.outputTokens.append([u"StartTag", token["name"], token["data"]])
+        self.outputTokens.append([u"StartTag", token["name"], dict(token["data"][::-1])])
 
     def processEmptyTag(self, token):
         if token["name"] not in constants.voidElements:
             self.outputTokens.append(u"ParseError")
-        self.outputTokens.append([u"StartTag", token["name"], token["data"]])
+        self.outputTokens.append([u"StartTag", token["name"], dict(token["data"][::-1])])
 
     def processEndTag(self, token):
-        if token["data"]:
-            self.processParseError(None)
         self.outputTokens.append([u"EndTag", token["name"]])
 
     def processComment(self, token):
@@ -71,7 +53,7 @@ class TokenizerTestParser(object):
         pass
 
     def processParseError(self, token):
-        self.outputTokens.append(u"ParseError")
+        self.outputTokens.append([u"ParseError", token["data"]])
 
 def concatenateCharacterTokens(tokens):
     outputTokens = []
@@ -87,26 +69,32 @@ def concatenateCharacterTokens(tokens):
     return outputTokens
 
 def normalizeTokens(tokens):
-    """ convert array of attributes to a dictionary """
     # TODO: convert tests to reflect arrays
-    for token in tokens:
-        if token[0] == 'StartTag':
-            token[2] = dict(token[2][::-1])
-    return tokens
+    for i, token in enumerate(tokens):
+        if token[0] == u'ParseError':
+            tokens[i] = token[0]
+    return simplejson.loads(simplejson.dumps(tokens))
 
-def tokensMatch(expectedTokens, recievedTokens):
+def tokensMatch(expectedTokens, receivedTokens, ignoreErrorOrder):
     """Test whether the test has passed or failed
 
-    For brevity in the tests, the test has passed if the sequence of expected
-    tokens appears anywhere in the sequence of returned tokens.
+    If the ignoreErrorOrder flag is set to true we don't test the relative
+    positions of parse errors and non parse errors
     """
-    return expectedTokens == recievedTokens
-    for i, token in enumerate(recievedTokens):
-        if expectedTokens[0] == token:
-            if (len(expectedTokens) <= len(recievedTokens[i:]) and
-                recievedTokens[i:i+len(expectedTokens)]):
-                return True
-    return False
+    if not ignoreErrorOrder:    
+        return expectedTokens == receivedTokens
+    else:
+        #Sort the tokens into two groups; non-parse errors and parse errors
+        tokens = {"expected":[[],[]], "received":[[],[]]}
+        for tokenType, tokenList in zip(tokens.keys(),
+                                         (expectedTokens, receivedTokens)):
+            for token in tokenList:
+                if token != "ParseError":
+                    tokens[tokenType][0].append(token)
+                else:
+                    tokens[tokenType][1].append(token)
+        
+        return tokens["expected"] == tokens["received"]
 
 
 class TestCase(unittest.TestCase):
@@ -118,36 +106,31 @@ class TestCase(unittest.TestCase):
             test['lastStartTag'] = None
         parser = TokenizerTestParser(test['contentModelFlag'], 
                                      test['lastStartTag'])
-            
-        tokens = normalizeTokens(parser.parse(test['input']))
+        tokens = parser.parse(test['input'])
         tokens = concatenateCharacterTokens(tokens)
         errorMsg = "\n".join(["\n\nContent Model Flag:",
                               test['contentModelFlag'] ,
-                              "\nExpected:", str(output), "\nRecieved:",
-                             str(tokens)])
-        self.assertEquals(tokensMatch(tokens, output), True, errorMsg)
-
-
-def test_tokenizer():
-    for filename in glob.glob('tokenizer/*.test'):
-        tests = simplejson.load(file(filename))
-        for test in tests['tests']:
-            yield (TestCase.runTokenizerTest, test)
+                              "\nInput:", str(test['input']),
+                              "\nExpected:", str(output),
+                              "\nreceived:", str(tokens)])
+        tokens = normalizeTokens(tokens)
+        ignoreErrorOrder = test.get('ignoreErrorOrder', False)
+        self.assertEquals(tokensMatch(tokens, output, ignoreErrorOrder), True,
+                          errorMsg)
 
 def buildTestSuite():
-    tests = 0
-    for func, test in test_tokenizer():
-        if 'contentModelFlags' not in test:
-            test["contentModelFlags"] = ["PCDATA"]
-        for contentModelFlag in test["contentModelFlags"]:
-            tests += 1
-            testName = 'test%d' % tests
-            test["contentModelFlag"] = contentModelFlag
-            testFunc = lambda self, method=func, test=test: \
-                method(self, test)
-            testFunc.__doc__ = "\t".join([test['description'], str(test['input'])])
-            instanceMethod = new.instancemethod(testFunc, None, TestCase)
-            setattr(TestCase, testName, instanceMethod)
+    for filename in html5lib_test_files('tokenizer', '*.test'):
+        tests = simplejson.load(file(filename))
+        testName = os.path.basename(filename).replace(".test","")
+        for index,test in enumerate(tests['tests']):
+            if 'contentModelFlags' not in test:
+                test["contentModelFlags"] = ["PCDATA"]
+            for contentModelFlag in test["contentModelFlags"]:
+                test["contentModelFlag"] = contentModelFlag
+                def testFunc(self, test=test):
+                    self.runTokenizerTest(test)
+                testFunc.__doc__ = "\t".join([testName, test['description']])
+                setattr(TestCase, 'test_%s_%d' % (testName, index), testFunc)
     return unittest.TestLoader().loadTestsFromTestCase(TestCase)
 
 def main():
